package com.bearprogrammer.blog.cs.datastructure;

public class ChainLink<E> {
	
	private ChainLink<E> next;
	private E object;
	
	public ChainLink() {
	}
	
	public ChainLink(E object) {
		this.object = object;
	}
	
	/**
	 * Check if this linked object chain contains the object specified.
	 * 
	 * @param object Object to check for.
	 * @return	True if the object is in the chain.
	 */
	public boolean contains(E object) {
		ChainLink<E> toCheck = this;
		while (toCheck != null) {
			if (toCheck.getObject().equals(object)) {
				return true;
			}
			toCheck = toCheck.getNext();
		}
		return false;
	}
	
	/**
	 * Return the last link in this chain.
	 * 
	 * @return The last link in this chain.
	 */
	public ChainLink<E> getLast() {
		ChainLink<E> toCheck = this;
		while (toCheck != null) {
			if (toCheck.getNext() == null) {
				break;
			}
			toCheck = toCheck.getNext();
		}
		return toCheck;
	}

	public ChainLink<E> getNext() {
		return next;
	}

	public E getObject() {
		return object;
	}
	
	/**
	 * Remove the link that contains the element from this chain and return the
	 * new first element for it.
	 * 
	 * @param toRemove
	 *            The object to search for.
	 * @return The first link in this chain or null if this chain has no more
	 *         links.
	 */
	public ChainLink<E> removeElement(E toRemove) {
		/*
		 * If this is the link that contains the object, return the next element
		 * in the chain, which can be null, meaning that this chain is now empty.
		 */
		if (getObject().equals(toRemove)) {
			return getNext();
		} else {
			ChainLink<E> previous = this;
			ChainLink<E> toCheck = getNext();
			while (toCheck != null) {
				// If found the link that contains the object
				if (toCheck.getObject().equals(toRemove)) {
					previous.setNext(toCheck.getNext());
					break;
				}
				
				previous = toCheck;
				toCheck = toCheck.getNext();
			}
			
			return this; // any other case, the chain begins where it used to
		}
	}

	public void setNext(ChainLink<E> next) {
		this.next = next;
	}

	public void setObject(E object) {
		this.object = object;
	}

}
