package com.bearprogrammer.blog.cs.datastructure;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import com.bearprogrammer.blog.cs.util.HashFunction;
import com.bearprogrammer.blog.cs.util.hashfunction.DivisionMethodHash;

public class HashTable<E> implements Set<E> {
	
	private class HashTableIterator implements Iterator<E> {
		
		private ChainLink<E> next = null;
		private ChainLink<E> lastReturned = null;
		
		public HashTableIterator() {
			next = findNext();
		}
		
		@SuppressWarnings("unchecked")
		private ChainLink<E> findNext() {
			ChainLink<E> result = ( next == null ? null : next.getNext() );
			if (result == null) {
				int startFrom = next == null ? 0 : hashElement(next.getObject()) + 1;
				for (int i = startFrom; i < table.length; i++) {
					if (table[i] != null) {
						result = (ChainLink<E>) table[i];
						break;
					}
				}
			}
			return result;
		}

		@Override
		public boolean hasNext() {
			return next != null;
		}

		@Override
		public E next() {
			lastReturned = next;
			next = findNext();
			return  lastReturned.getObject();
		}

		@Override
		public void remove() {
			ChainLink<E> toRemove = lastReturned;
			HashTable.this.remove(toRemove.getObject());
		}
	}
	
	private HashFunction hashFunction = new DivisionMethodHash();
	
	private int size = 0;
	
	@SuppressWarnings("unchecked")
	private ChainLink<Object>[] table = new ChainLink[Byte.MAX_VALUE];
	
	@Override
	public boolean add(E element) {
		if (element == null) {
			throw new NullPointerException("Can't add a null element.");
		}
		
		boolean added = false;
		
		int hash = hashElement(element);
		ChainLink<Object> object = table[hash];
		
		if (object == null) {
			table[hash] = new ChainLink<Object>(element);
			added = true;
		} else {
			if (!object.contains(element)) {
				object.getLast().setNext(new ChainLink<Object>(element));
				added = true;
			}
		}
		
		if (added == true) {
			size++;
		}
		
		return added;
	}

	private int hashElement(Object element) {
		int hash = hashFunction.hashObject(element.hashCode());
		return hash;
	}

	@Override
	public boolean addAll(Collection<? extends E> all) {
		boolean changed = false;
		for (E element : all) {
			changed |= add(element);
		}
		return changed;
	}

	@Override
	@SuppressWarnings("unchecked")
	public void clear() {
		table = new ChainLink[Byte.MAX_VALUE];
	}

	@Override
	public boolean contains(Object element) {
		int hash = hashElement(element);
		ChainLink<Object> link = table[hash];
		if (link != null) {
			return link.contains(element);
		}
		return false;
	}

	@Override
	public boolean containsAll(Collection<?> all) {
		boolean containAll = false;
		for (Object o : all) {
			containAll |= contains(o);
		}
		return containAll;
	}

	@Override
	public boolean isEmpty() {
		return size == 0;
	}

	@Override
	public Iterator<E> iterator() {
		return new HashTableIterator();
	}

	@Override
	public boolean remove(Object element) {
		boolean removed = false;
		
		if (element != null) {
			int hash = hashElement(element);
			ChainLink<Object> link = table[hash];
			
			if (link != null && link.contains(element)) {
				table[hash] = link.removeElement(element);
				removed = true;
				size--;
			}
		}
		
		return removed;
	}

	@Override
	public boolean removeAll(Collection<?> all) {
		boolean changed = false;
		for (Object o : all) {
			changed |= remove(o);
		}
		return changed;
	}

	@Override
	public boolean retainAll(Collection<?> all) {
		boolean changed = false;
		for (Iterator<E> iterator = this.iterator(); iterator.hasNext();) {
			E element = iterator.next();
			if (!all.contains(element)) {
				iterator.remove();
				changed = true;
			}
		}
		
		return changed;
	}

	@Override
	public int size() {
		return size;
	}

	@Override
	public Object[] toArray() {
		Object [] result = new Object[size];
		int counter = 0;
		for (E element : this) {
			result[counter] = element;
			counter++;
		}
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public <T> T[] toArray(T[] array) {
		if (array.length != size) {
			throw new IllegalArgumentException("Provided array should have same size as HashTable. Provided size: " + array.length + ", HashTable size: " + size);
		}
		
		int counter = 0;
		for (E element : this) {
			array[counter] = (T) element;
			counter++;
		}
		
		return array;
	}

}
