package com.bearprogrammer.blog.cs.sort;

import java.util.Arrays;

/**
 * Implementation of merge sort that uses recursion.
 * 
 * @author Vinicius Isola
 * 
 * @param <E>
 *            Type to be sorted.
 * @see http://en.wikipedia.org/wiki/Merge_sort
 */
public class MergeSort<E extends Comparable<E>> implements SortAlgorithm<E> {

	@Override
	public void sort(E[] toSort) {
		// If one or none elements, we're done
		if (toSort.length <= 1) {
			return;
		}

		// Get the middle of the array
		int mid = toSort.length / 2;

		// Split it into two smaller arrays
		E[] leftArray = Arrays.copyOfRange(toSort, 0, mid);
		E[] rightArray = Arrays.copyOfRange(toSort, mid, toSort.length);

		// Recursively sort these arrays
		sort(leftArray);
		sort(rightArray);

		// Now merge them
		int i = 0, j = 0, k = 0;
		
		// While haven't used all elements of both arrays
		while (i < leftArray.length || j < rightArray.length) {
			
			// Default comparison
			int comparison = 0;
			
			/*
			 * Compare left to right if we still have elements in both arrays to
			 * be compared.
			 */
			if (i < leftArray.length && j < rightArray.length) {
				comparison = leftArray[i].compareTo(rightArray[j]);
			}

			// Use element from left array
			if (i < leftArray.length && comparison <= 0) {
				toSort[k] = leftArray[i];
				i++;
				
			// Use element from right array
			} else {
				toSort[k] = rightArray[j];
				j++;
			}
			
			// Move to next position in final array
			k++;
		}
	}

}
