package com.bearprogrammer.blog.cs.sort.iterative;

import java.util.Arrays;

/**
 * Abstract implementation of the basic functionality of a sorting algorithm.
 * 
 * @author Vinicius Isola
 * 
 * @param <E>
 *            The type of element to be sorted.
 */
public abstract class AbstractIterativeSortAlgorithm<E extends Comparable<E>> implements IterativeSortAlgorithm<E> {
	
	/**
	 * Step counter. Subclasses should call {@link #incrementSteps()} to
	 * increment this.
	 */
	protected int steps = 0;
	
	/**
	 * The array of data to be sorted.
	 */
	protected E [] toSort;
	
	@Override
	public int getSteps() {
		return steps;
	}
	
	@Override
	public E[] getToSort()  {
		return Arrays.copyOf(toSort, toSort.length);
	}
	
	/**
	 * Increments the {@link #steps} variable.
	 */
	protected void incrementSteps() {
		steps++;
	}
	
	@Override
	public void setToSort(E[] toSort) {
		steps = 0;
		this.toSort = Arrays.copyOf(toSort, toSort.length);
	}

}
