package com.bearprogrammer.blog.cs.sort.iterative;

/**
 * Iterative implementation of the bubble sort algorithm.
 * 
 * @author Vinicius Isola
 * 
 * @param <E>
 *            The type of elements that will be sorted.
 * @see http://en.wikipedia.org/wiki/Bubble_sort
 */
public class BubbleSort<E extends Comparable<E>> extends AbstractIterativeSortAlgorithm<E> {
	
	/**
	 * Will mark the last position where a check/swap happened.
	 */
	int lastPosition = 0;
	
	/**
	 * Count how many swaps happened in the last iteration.
	 */
	int swaps = -1;
	
	@Override
	public boolean hasNextStep() {
		// When no more swaps happened, we're finished
		return toSort.length != 1 && ! (lastPosition == 0 && swaps == 0);
	}

	@Override
	public ComparisonPair<E> nextStep() {
		incrementSteps();
		
		// If we are starting a new pass through, reset swaps
		if (lastPosition == 0) {
			swaps = 0;
		}
		
		// Element that will be checked in this iteration
		E thisElement = toSort[lastPosition];
		
		// Element to compare to
		E nextElement = toSort[lastPosition + 1];
		
		// If next elements is greater than this element, swap
		if (thisElement.compareTo(nextElement) > 0) {
			toSort[lastPosition] = nextElement;
			toSort[lastPosition + 1] = thisElement;
			swaps ++;
		}
		
		lastPosition ++;
		
		// If we got to the end of the array, start over
		if (lastPosition >= toSort.length - 1) {
			lastPosition = 0;
		}
		
		return new ComparisonPair<E>(thisElement, nextElement);
	}
	
	@Override
	public void setToSort(E[] toSort) {
		super.setToSort(toSort);
		
		// Reset variables if we change the array
		lastPosition = 0;
		swaps = -1;
	}

}