package com.bearprogrammer.blog.cs.sort.iterative;

/**
 * A comparison pair holder. This class holds two elements that will be or were
 * compared in a {@link IterativeSortAlgorithm sorting algorithm}.
 * 
 * @author Vinicius Isola
 * 
 * @param <E>
 *            Type of the element to be compared.
 */
public class ComparisonPair<E extends Comparable<E>> {

	/**
	 * The element that is being processed by the algorithm.
	 */
	final E thisElement;
	
	/**
	 * The element that the element will be compared to.
	 */
	final E comparingTo;
	
	/**
	 * The comparison result between the two.
	 */
	final int comparison;

	/**
	 * Create a new comparison pair.
	 * 
	 * @param thisElement
	 *            The element that is being processed.
	 * @param comparingTo
	 *            The element to compare to.
	 */
	public ComparisonPair(E thisElement, E comparingTo) {
		super();
		this.thisElement = thisElement;
		this.comparingTo = comparingTo;
		this.comparison = thisElement.compareTo(comparingTo);
	}

	/**
	 * The element to compare to.
	 * 
	 * @return The element to compare to.
	 */
	public E getComparingTo() {
		return comparingTo;
	}

	/**
	 * The result of this comparison.
	 * 
	 * @return The result.
	 */
	public int getComparison() {
		return comparison;
	}

	/**
	 * The element that is being processed.
	 * 
	 * @return The element that will be processed.
	 */
	public E getThisElement() {
		return thisElement;
	}
	
	@Override
	public String toString() {
		return thisElement + " comparing " + comparingTo;
	}

}