package com.bearprogrammer.blog.cs.sort.iterative;


/**
 * An iterative representation of a sorting algorithm. It can be used to easily
 * demonstrate how a specific algorithm works detailing how many steps it used
 * and what is the state of the array being sorted between two steps.
 * 
 * @author Vinicius Isola
 * 
 * @param <E>
 *            The type of elements that will be sorted.
 */
public interface IterativeSortAlgorithm<E extends Comparable<E>> {

	/**
	 * Get the array being sorted as it is in this iteration.
	 * 
	 * @return The array being sorted.
	 */
	E[] getToSort();

	/**
	 * The numbers of steps executed so far.
	 * 
	 * @return The numbers of steps executed.
	 */
	int getSteps();

	/**
	 * Check if this algorithm has a next step to be executed or not.
	 */
	boolean hasNextStep();

	/**
	 * Execute the next step of the iteration and return the element that was
	 * processed in this iteration.
	 * 
	 * @return The element processed in the iteration.
	 */
	ComparisonPair<E> nextStep();

	/**
	 * Set the array to be sorted.
	 * 
	 * @param toSort
	 *            The array to be sorted.
	 */
	void setToSort(E[] toSort);

}