package com.bearprogrammer.blog.cs.sort.iterative;

/**
 * An iterative implementation of the selection sort algorithm.
 * 
 * @author Vinicius Isola
 * 
 * @param <E>
 *            Type of element to be sorted.
 * @see http://en.wikipedia.org/wiki/Selection_sort
 */
public class SelectionSort<E extends Comparable<E>> extends AbstractIterativeSortAlgorithm<E> {
	
	/**
	 * Position processed.
	 */
	int lastPosition = 0;
	
	/**
	 * Position of the last element compared to.
	 */
	int compareTo = 0;
	
	/**
	 * Position of the last minimum found.
	 */
	int lastMinimum = 0;

	@Override
	public boolean hasNextStep() {
		// We stop when all positions were compared
		return toSort.length != 1 && (steps == 0 || lastPosition < toSort.length - 2);
	}

	@Override
	public ComparisonPair<E> nextStep() {
		incrementSteps();
		
		// Move to next position to compare
		compareTo++;
		
		// Element that is being processed
		E thisElement = toSort[lastPosition];
		
		// If we reached the end of the array to compare to
		if (compareTo >= toSort.length) {
			// If we have a good minimum, swap it
			if (thisElement.compareTo(toSort[lastMinimum]) > 0) {
				swap();
			}
			
			// Reset counters
			lastPosition++;
			compareTo = lastPosition + 1;
			lastMinimum = lastPosition;
			
			// Get the new this element
			thisElement = toSort[lastPosition];
		}
		
		// Element to compare to
		E compareToElement = toSort[compareTo];
		// Initialize minimum
		E minimum = toSort[lastMinimum];
		
		// If we found a better minimum, point to it
		if (thisElement.compareTo(compareToElement) > 0 && minimum.compareTo(compareToElement) >= 0) {
			lastMinimum = compareTo;
			
			/*
			 * We can get to a situation where we are done but haven't swapped the
			 * element yet so we need to do this check here too.
			 */
			if (!hasNextStep()) {
				swap();
			}
		}
		
		return new ComparisonPair<E>(thisElement, compareToElement);
	}

	/**
	 * Swap the last position with the last minimum found so far.
	 */
	private void swap() {
		E temp = toSort[lastPosition];
		toSort[lastPosition] = toSort[lastMinimum];
		toSort[lastMinimum] = temp;
	}
	
	@Override
	public void setToSort(E[] toSort) {
		super.setToSort(toSort);
		lastPosition = 0;
		compareTo = 0;
	}

}
