package com.bearprogrammer.blog.cs.util;

/**
 * Represent a hash function that can be used to hash any object into an integer
 * that should be contained into a specific interval.
 * 
 * @author Vinicius Isola
 */
public interface HashFunction {

	/**
	 * Return the maximum that is being used by this hash function.
	 * 
	 * @return The maximum being used.
	 */
	public int getMaximum();

	/**
	 * Hash the object to an integer within the specified interval.
	 * 
	 * @param obj
	 *            The object to be hashed.
	 * @return The hash for the specified object.
	 */
	public int hashObject(Object obj);

	/**
	 * Change the max for this hashing function.
	 * 
	 * @param max
	 *            The new max.
	 */
	public void setMaximum(int max);

}
