package com.bearprogrammer.blog.cs.util.hashfunction;

import com.bearprogrammer.blog.cs.util.PrimeNumber;

/**
 * <p>
 * A hash function that uses the division method using a big prime to get a good
 * distribution.
 * </p>
 * 
 * <p>
 * Algorithms used in this class were taken from <a href="http
 * ://www.fearme.com/misc/alg/node29.html">here</a>.
 * </p>
 * 
 * @author Vinicius Isola
 */
public class DivisionMethodHash extends AbstractHashFunction {
	
	int bigPrime = 1;
	
	public DivisionMethodHash() {
		resetBigPrime();
	}

	@Override
	public int hashObject(Object obj) {
		return Math.abs(( obj.hashCode() % bigPrime ) % getMaximum());
	}
	
	/**
	 * Change the prime number that is used to hash accordingly to the maximum limit to be used for the hash.
	 */
	private void resetBigPrime() {
		bigPrime = PrimeNumber.getNextPrime(getMaximum() * (int) Math.pow(10, Integer.toString(getMaximum()).length()));
	}
	
	@Override
	public void setMaximum(int max) {
		super.setMaximum(max);
		resetBigPrime();
	}

}
