package com.bearprogrammer.blog.cs.sort;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.ServiceLoader;

import junit.framework.Assert;

import org.junit.Test;

public class SortAlgorithmTest {
	
	private List<SortAlgorithm<Integer>> algorithms;
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public SortAlgorithmTest() {
		this.algorithms = new ArrayList<SortAlgorithm<Integer>>();
		ServiceLoader<SortAlgorithm> loader = ServiceLoader.load(SortAlgorithm.class);
		for (SortAlgorithm algorithm : loader) {
			algorithms.add(algorithm);
		}
	}
	
	private Integer [] getRandomNumbers(int seed, int count) {
		Integer [] randomNumbers = new Integer[count];
		Random random = new Random(seed);
		for (int j = 0; j < count; j++) {
			randomNumbers[j] = random.nextInt(5000) + 1;
		}
		return randomNumbers;
	}
	
	@Test
	public void algorithmShouldSortNumbers() {
		for (SortAlgorithm<Integer> algorithm : algorithms) {
			for (int times = 0; times <= 10; times++) {
				int count = (int) Math.pow(3, times);
				Integer [] randomNumbers = getRandomNumbers(times, count);
				Integer [] clone = Arrays.copyOf(randomNumbers, randomNumbers.length);
				Arrays.sort(clone);

				long start = System.nanoTime();
				algorithm.sort(randomNumbers);
				long end = System.nanoTime();
				
				for (int i = 0; i < randomNumbers.length - 1; i++) {
					String message = String.format("Should be sorted,  expected: %1$d <= %2$d, Algorithm: %3$s", randomNumbers[i], randomNumbers[i + 1], algorithm.getClass().getSimpleName());
					Assert.assertTrue(message, randomNumbers[i] <= randomNumbers[i+1]);
					Assert.assertEquals("Should not have lost any elements.", clone[i], randomNumbers[i]);
				}
				
				System.out.printf(">>>>>>>>> Algorithm: %1$s, finished %2$d element(s) in %3$,d ns.\n", algorithm.getClass().getSimpleName(), count, end - start);
			}
		}
	}
	

}
