package com.bearprogrammer.blog.cs.sort.iterative;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.ServiceLoader;

import junit.framework.Assert;

import org.junit.Test;

import com.bearprogrammer.blog.cs.sort.iterative.IterativeSortAlgorithm;

public class IterativeSortAlgorithmTest {
	
	private List<IterativeSortAlgorithm<Integer>> algorithms;
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public IterativeSortAlgorithmTest() {
		this.algorithms = new ArrayList<IterativeSortAlgorithm<Integer>>();
		ServiceLoader<IterativeSortAlgorithm> loader = ServiceLoader.load(IterativeSortAlgorithm.class);
		for (IterativeSortAlgorithm algorithm : loader) {
			algorithms.add(algorithm);
		}
	}
	
	private Integer [] getRandomNumbers(int seed, int count) {
		Integer [] randomNumbers = new Integer[count];
		Random random = new Random(seed);
		for (int j = 0; j < count; j++) {
			randomNumbers[j] = random.nextInt(5000) + 1;
		}
		return randomNumbers;
	}
	
	@Test
	public void algorithmShouldSortNumbers() {
		for (IterativeSortAlgorithm<Integer> algorithm : algorithms) {
			System.out.println(">>>>>>>>> Algorithm: " + algorithm.getClass().getSimpleName());
			for (int times = 0; times <= 10; times++) {
				int count = (int) Math.pow(2, times);
				int maxIterations = count * count + 1;
				
				algorithm.setToSort(getRandomNumbers(times, count));
				
				while (algorithm.hasNextStep()) {
					algorithm.nextStep();
					if (algorithm.getSteps() > maxIterations) {
						Assert.fail("Too many iterations: " + algorithm.getSteps() + ", Algorithm: " + algorithm.getClass().getSimpleName());
					}
				}
				
				Integer [] randomNumbers = algorithm.getToSort();
				for (int i = 0; i < randomNumbers.length - 1; i++) {
					Assert.assertTrue("Should be sorted, expected: " + randomNumbers[i] + " <= " + randomNumbers[i+1] + ", Algorithm: " + algorithm.getClass().getSimpleName(), 
							randomNumbers[i] <= randomNumbers[i+1]);
				}
				
				System.out.println("Array sorted in " + algorithm.getSteps() + " steps for " + count + " elements.");
			}
		}
	}
	

}
