package com.bearprogrammer.blog.cs.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.ServiceLoader;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

public class HashFunctionTest {

	private static final int ITERATIONS = 10000;
	private static final int MAX_DIFFERENCE = (Byte.MAX_VALUE - Byte.MIN_VALUE) / 4;
	
	private List<HashFunction> hashFunctions = new ArrayList<HashFunction>();
	
	@Before
	public void setup() {
		ServiceLoader<HashFunction> loader = ServiceLoader.load(HashFunction.class);
		for (HashFunction function : loader) {
			hashFunctions.add(function);
		}
	}
	
	@Test
	public void basicHashTest() {
		for (HashFunction function : hashFunctions) {
			Random random = new Random(1);
			for (int i = 0; i < ITERATIONS; i++) {
				int number = random.nextInt();
				int hash = function.hashObject(number);
	
				Assert.assertTrue("Hash should be greater than or equal to zero. It was: " + hash, hash >= 0);
				Assert.assertTrue("Hash should be less than or equal to byte maximum. It was: " + hash, hash <= Byte.MAX_VALUE);
			}
		}
	}
	
	@Test
	public void hashShouldBeEvenlyDistributed() {
		for (HashFunction function : hashFunctions) {
			Random random = new Random(1);
			Map<Integer, Integer> hashes = new HashMap<Integer, Integer>(); 
			for (int i = 0; i < ITERATIONS; i++) {
				int number = random.nextInt();
				int hash = function.hashObject(number);
				
				Integer count = hashes.get(hash);
				if (count == null) {
					hashes.put(hash, 1);
				} else {
					hashes.put(hash, count + 1);
				}
			}
			
			int min = Integer.MAX_VALUE;
			int max = 0;
			for (Integer count : hashes.values()) {
				if (count < min) min = count;
				if (count > max) max = count;
			}
			
			Assert.assertTrue("Should have generated more than one hash.", hashes.size() > 1);
			Assert.assertTrue("Hash should evenly distribute it's hashes. Difference was: " + (max - min), max - min <= MAX_DIFFERENCE);
		}
	}

}
