package com.bearprogrammer.blog.cs.util;

import junit.framework.Assert;

import org.junit.Test;

public class PrimeNumberTest {
	
	private static final int [] PRIME_NUMBERS = {1, 2, 3, 5, 7, 11, 13, 17, 19, 23, 29};
	private static final int [] NON_PRIME_NUMBERS = {4, 6, 8, 9, 10, 12, 14, 15, 16, 18, 20, 21, 22, 24, 25, 26, 27, 28, 30};
	
	@Test
	public void compareSpeed () {
		int count = 1000;
		int totalSlow = 0;
		int totalFast = 0;
		for (int i = 1; i <= count; i++) {
			long start = System.nanoTime();
			PrimeNumber.isPrime(i);
			totalFast += System.nanoTime() - start;
			
			start = System.nanoTime();
			PrimeNumber.slowIsPrime(i);
			totalSlow += System.nanoTime() - start;
		}
		System.out.println("Fast average: " + (totalFast/count) + ", Slow average: " + (totalSlow/count));
	}
	
	@Test
	public void performanceToGetVeryHighPrime() {
		PrimeNumber primeNumber = new PrimeNumber(15485863);
		
		long start = System.currentTimeMillis();
		for (int i = 0; i < 10000; i++) {
			primeNumber.getNextPrime();
		}
		System.out.println( (System.currentTimeMillis() - start) + "ms to get to " + primeNumber.getLastFound());
	}
	
	@Test
	public void getNextPrimeIteratorShouldWork() {
		PrimeNumber primeNumber = new PrimeNumber();
		for (int i = 1; i < PRIME_NUMBERS.length; i++) {
			Assert.assertSame("Should return next prime number.", PRIME_NUMBERS[i], primeNumber.getNextPrime());
		}
	}
	
	@Test
	public void getNextPrimeShouldWork() {
		for (int i = 0; i < PRIME_NUMBERS.length - 1; i++) {
			Assert.assertSame("Should return next prime number.", PRIME_NUMBERS[i + 1], PrimeNumber.getNextPrime(PRIME_NUMBERS[i]));
		}
	}
	
	@Test
	public void isPrimeNumberShouldWork() {
		for (int prime : PRIME_NUMBERS) {
			Assert.assertTrue(prime + " is prime", PrimeNumber.isPrime(prime));
		}
		
		for (int nonPrime : NON_PRIME_NUMBERS) {
			Assert.assertFalse(nonPrime + " is not prime", PrimeNumber.isPrime(nonPrime));
		}
		
		for (int i = 2; i < PRIME_NUMBERS.length - 1; i++) {
			int number = PRIME_NUMBERS[i] * PRIME_NUMBERS[i + 1];
			Assert.assertFalse(number + " is not prime", PrimeNumber.isPrime(number));
		}
	}

}
